package newsTicker;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Panel;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

public class NewsTicker extends Panel {

	private int 				geschwindigkeit;
	private char 				laufrichtung;
	private Nachricht[] 		nachrichten;
	private TickerThread 		tickerSchleife;
	private OeffneLinkListener 	listener;

	// Konstruktor
	public NewsTicker( int breite, int hoehe, Nachricht[] nachrichten, int geschwindigkeit, char laufrichtung ) {
		
		// Datenuebernahme
		this.nachrichten = nachrichten;
		this.geschwindigkeit = geschwindigkeit;
		this.laufrichtung = laufrichtung;
		
		// Panel-Grundkonfiguration
		setBackground( Color.WHITE );
		setSize( breite, hoehe );

		// Alle Nachrichten ausrichten
		resetPositionen();
		
		// tickerSchleife starten  ( Timer: warten / Nachrichten verschieben / warten / verschieben / ... )
		tickerSchleife = new TickerThread( this.geschwindigkeit );
		tickerSchleife.start();
		
		// 	Lauschen auf Maus-Ereignisse -> mausKlick() starten
		this.addMouseListener( new MouseAdapter() { 
				public void mouseReleased( MouseEvent e ) { 
					// 	Starte Methode mausKlick des geklickten NewsTickers
					NewsTicker newsticker = (NewsTicker) e.getComponent();
					newsticker.mausKlick( e.getX(), e.getY() );
				}	
			}
		);

	}
	
	public void  	update( Graphics screen ) {
		// Ueberschreiben der Standard update()-Methode: Panel _nicht_ loeschen
		paint( screen );
	}
	public void  	paint( Graphics screen ) {
		// Fuer DoubleBuffer (gegen flimmern)
		Image offscreenImg = createImage( getWidth(), getHeight() );
		Graphics offscreen = offscreenImg.getGraphics();
		// Fuer alle Nachrichten:
		for (int nr = 0; nr < nachrichten.length; nr++) {
			//	Schriftzug der Nachricht auf dem offscreen-Objekt zeichnen (DoubleBuffering)
			offscreen.setFont( new Font( nachrichten[nr].getSchriftArt(), nachrichten[nr].getSchriftStil(), nachrichten[nr].getSchriftGroesse() ) );
			offscreen.setColor( nachrichten[nr].getSchriftFarbe() );
			offscreen.drawString( nachrichten[nr].getTitel(), nachrichten[nr].getX(), nachrichten[nr].getY() );
		}
		// offscreen-Objekt auf Panel kopieren
		screen.drawImage( offscreenImg, 0, 0, this );
	}
		
	public void  	mausKlick(int x, int y) {
		boolean keinHit = true;
		// Gehe durch alle Nachrichten,
		for ( int nr=0; nr < nachrichten.length; nr++ ) {
			// und pruefe jede, ob sie "getroffen"/geklickt wurde
			if ( nachrichten[nr].treffer( x, y )) {
				keinHit = false;
				// OeffneLink-Ereigniss erzeugen: eine Nachricht wurde angeklickt/getroffen
				listener.oeffneLink( 
					new OeffneLinkEvent( this, nachrichten[nr].getLink() ) 
				);
				nachrichten[nr].setSchriftFett(true);
			} else { 
				nachrichten[nr].setSchriftFett(false);
			}
		}
		if ( keinHit) {
			if ( x < 32 ) { langsamer(); } 
			if ( x > 10  && x < (getBreite()-32) ) { toggleLaufrichtung(); }
			if ( x > (getBreite()-32) ) { schneller(); } 
		}
	}
	
	public void 	addOeffneLinkListener(OeffneLinkListener listener) {
		this.listener = listener;
	}
	
	public void	 	setLaufrichtung(char laufrichtung) {
		this.laufrichtung = laufrichtung;
	}
	public char  	getLaufrichtung() {
		return laufrichtung;
	}
	public void  	toggleLaufrichtung() {
		switch (laufrichtung) {
			case 'v': laufrichtung = 'h'; break;
			case 'h': laufrichtung = 'v'; break;
		}
		resetPositionen();
		repaint();
	}

	public void  	resetPositionen() {
		switch ( laufrichtung ) {
		case 'h':
			nachrichten[0].setPosition( getWidth(),  getHeight()/2 + nachrichten[0].getHoehe()/2 );
			for (int nr=1; nr < nachrichten.length; nr++) {
				nachrichten[nr].setPosition( nachrichten[nr-1].getX()+nachrichten[nr-1].getBreite()+Standard.abstandX, 
											 getHeight()/2 + nachrichten[nr].getHoehe()/2 );
			}
			break;
		case 'v': 
			nachrichten[0].setPosition( 16,  0 );
			for (int nr=1; nr < nachrichten.length; nr++) {
				nachrichten[nr].setPosition( 16,  nachrichten[nr-1].getY() - nachrichten[nr-1].getHoehe() - Standard.abstandY );
			}
			break;
		}
	}
	public void  	verschiebeNachrichten() {
		switch ( laufrichtung ) {	
			case 'h':	
				// alle nachrichten horizontal nach links verschieben
				for (int nr=0; nr < nachrichten.length; nr++) { nachrichten[nr].verschieben( -1, 0 ); } 
				// wenn die letzte nachricht links raus ist muessen alle positionen zurueckgesetzt werden 
				if ( nachrichten[ nachrichten.length-1 ].getX() + nachrichten[ nachrichten.length-1 ].getBreite()  <   0 ) 
					resetPositionen();
				break;
			case 'v':	
				// alle nachrichten vertikal nach unten verschieben
				for (int nr=0; nr < nachrichten.length; nr++) { nachrichten[nr].verschieben( 0, +1); }
				// wenn die letzte nachricht unten raus ist muessen alle positionen zurueckgesetzt werden 
				if ( nachrichten[nachrichten.length-1].getY() - nachrichten[nachrichten.length-1].getHoehe() > getHeight() )
					resetPositionen();
				break;
		}
		// Neuzeichnen...
		repaint();
	}
	
	public void 	schneller() {
		if ( geschwindigkeit < 9 ) { 
			geschwindigkeit += 1;
			tickerSchleife.setGeschwindigkeit(geschwindigkeit);
		}
	}
	public void 	langsamer() {	
		if ( geschwindigkeit > 1 ) { 
			geschwindigkeit -= 1;
			tickerSchleife.setGeschwindigkeit(geschwindigkeit);
		}
	}
	
	public int 		getBreite() {
		return getWidth();
	}
	public int 		getHoehe() {
		return getHeight();
	}
	
	public void  	start() {
		tickerSchleife.anhalten(false);			
	}
	public void  	stop() {
		tickerSchleife.anhalten(true);		
	}

	public class  TickerThread extends Thread {

		private boolean pauseFlag = true;
		private int	wartezeit;
		
		public TickerThread( int geschwindigkeit ) {
			setGeschwindigkeit( geschwindigkeit );
			setName("::: TickerThread ( warten & verschieben ) :::");
		}
		
		public void 	run() {
			while ( true ) {
				if ( !pauseFlag ) { // wenn  nicht pausiert, dann verschiebe (nach einer wartezeit)
					try { Thread.sleep( wartezeit ); } // warte je nach geschwindigkeit u. richtung
					catch( InterruptedException e) { return; }
					verschiebeNachrichten();
				} else {	// sonst warte
					try { Thread.sleep( 100 ); } 
					catch( InterruptedException e) { return; }
				}
			}
		}
		public void 	anhalten( boolean pauseFlag ) {
			this.pauseFlag = pauseFlag;
		}
		public void 	setGeschwindigkeit(int geschwindigkeit) {
			switch (geschwindigkeit) {
				case 1:  	wartezeit =111;  break;
				case 2:  	wartezeit = 88;  break;
				case 3:  	wartezeit = 66;  break;
				case 4:  	wartezeit = 44;  break;
				case 5:  	wartezeit = 33;  break;
				case 6:  	wartezeit = 22;  break;
				case 7:  	wartezeit = 11;  break;
				case 8:  	wartezeit =  7;  break;
				case 9:  	wartezeit =  4;  break;
				default: 	setGeschwindigkeit( Standard.geschwindigkeit ); break;
			}
		}
	}
}